﻿using Microsoft.AspNetCore.Http;
using System.IO;
using System.Text;
using System.Threading.Tasks;

namespace Microsoft.AspNetCore.Builder
{
    // You may need to install the Microsoft.AspNetCore.Http.Abstractions package into your project
    public class GatewayMiddleware
    {
        private readonly RequestDelegate _next;

        public GatewayMiddleware(RequestDelegate next)
        {
            _next = next;
        }

        public async Task Invoke(HttpContext httpContext)
        {
            //支持类似支付宝的网关模式
            //公共请求参数
            //app_id method format charset sign_type sign timestamp version app_auth_token biz_content

            //请求参数(业务)，建议json格式
            //trade_no  out_trade_no operator_id
            if (httpContext.Request.Path.ToString().ToLower() == "/gateway"|| httpContext.Request.Path.ToString().ToLower() == "/gateway.do")
            {
                //调用方post form提交，获取公共请求参数，处理做转发
                if (httpContext.Request.Method.ToLower() == "get")
                {
                    httpContext.Response.ContentType = "text/plain;charset=utf-8";
                    await httpContext.Response.WriteAsync("调试错误，请回到请求来源地，重新发起请求");
                    return;
                }
                var reqForm = httpContext.Request.Form;
                if (reqForm == null || reqForm.Count == 0)
                {
                    httpContext.Response.ContentType = "text/plain;charset=utf-8";
                    await httpContext.Response.WriteAsync("调试错误，请回到请求来源地，重新发起请求");
                    return;
                }

                var app_id = reqForm.ContainsKey("appid") ? reqForm["appid"].ToString() : "";
                var method = reqForm.ContainsKey("method") ? reqForm["method"].ToString() : "";//接口名称（格式：模块.控制器.方法） 比如demo1.Values.SaveOrder
                var format = reqForm.ContainsKey("format") ? reqForm["format"].ToString() : "json";
                var charset = reqForm.ContainsKey("charset") ? reqForm["charset"].ToString() : "utf-8";
                var sign_type = reqForm.ContainsKey("sign_type") ? reqForm["sign_type"].ToString() : "md5";
                var sign = reqForm.ContainsKey("sign") ? reqForm["sign"].ToString() : "";
                var timestamp = reqForm.ContainsKey("timestamp") ? reqForm["timestamp"].ToString() : "";
                var version = reqForm.ContainsKey("version") ? reqForm["version"].ToString() : "";
                var app_auth_token = reqForm.ContainsKey("app_auth_token") ? reqForm["app_auth_token"].ToString() : "";
                var biz_content = reqForm.ContainsKey("biz_content") ? reqForm["biz_content"].ToString() : "";//业务接口参数 json格式

                
                //通过method参数拆分出 模块 控制器 方法
                var methods = method.Split('.');
                var moduleName = method.Length > 0 ? methods[0] : "";
                var controllerName = method.Length > 1 ? methods[1] : "";
                var actionName = method.Length > 2 ? methods[2] : "";

                //区分有版本和无版本两种情况，version不传或传空就是无版本
                var nextPath = string.IsNullOrEmpty(version) ? $"/{moduleName}/api/{controllerName}/{actionName}" : $"/{moduleName}/api/v{version}/{controllerName}/{actionName}";

                //下游业务接口暂时只支持post json格式的请求
                byte[] postData = Encoding.GetEncoding(charset).GetBytes(biz_content);
                httpContext.Request.Path = nextPath;
                httpContext.Request.ContentType = "application/json";
                httpContext.Request.ContentLength = postData.Length;
                httpContext.Request.Body = new MemoryStream(postData);                

                await _next(httpContext);
            }
            else
            {
                await _next(httpContext);
            }
        }
    }

    // Extension method used to add the middleware to the HTTP request pipeline.
    public static class GatewayMiddlewareExtensions
    {
        public static void UseGatewayMiddleware(this IApplicationBuilder app)
        {
            app.UseMiddleware<GatewayMiddleware>();
        }
    }
}
